#ifndef __P80211_H_
#define __P80211_H_
// system includes

#include <asm/types.h>

//local includes
#include "llist.h"

// standard protocol parameter lengths and limits
#define		WLAN_ADDR_LEN		6	// 6 bytes
#define 	WLAN_FCS_LEN		4	// 4 bytes
#define 	WLAN_WEP_IV_LEN		3	// 3 bytes
#define 	WLAN_SSID_MAXLEN	32	// 32 bytes
#define 	WLAN_DATA_MAXLEN	2312	// 2312 bytes

#define 	WLAN_1ADDR_HDR_LEN	10
#define 	WLAN_2ADDR_HDR_LEN	16
#define 	WLAN_3ADDR_HDR_LEN	24
#define		WLAN_4ADDR_HDR_LEN	30

// 802.11 protocol version
#define		WLAN_PROTO_VER		0

// 802.11 frame types
#define 	WLAN_FTYPE_MGMT		0x00	// management
#define 	WLAN_FTYPE_CTRL		0x01	// control
#define 	WLAN_FTYPE_DATA		0x02	// data
#define		WLAN_FTYPE_RESV		0x03	// reserved

// Management frame subtypes
#define 	WLAN_MGMT_AS_REQ		0x00	// association request
#define		WLAN_MGMT_AS_RES		0x01	// association response
#define 	WLAN_MGMT_REAS_REQ		0x02	// reassociation request
#define 	WLAN_MGMT_REAS_RES		0x03	// reassociation response
#define 	WLAN_MGMT_PROB_REQ		0x04	// probe request
#define 	WLAN_MGMT_PROB_RES		0x05	// probe response
#define 	WLAN_MGMT_RESV1			0x06	// reserved
#define 	WLAN_MGMT_RESV2			0x07	// reserved
#define 	WLAN_MGMT_BEACON		0x08	// beacon 
#define 	WLAN_MGMT_ATIM			0x09	// announcement traffic indication message(ATIM)
#define 	WLAN_MGMT_DISAS			0x0A	// disassociation
#define 	WLAN_MGMT_AUTH			0x0B	// authentication
#define 	WLAN_MGMT_DEAUTH		0x0C	// deauthentication
#define 	WLAN_MGMT_RESV3			0x0D	// reserved	
#define 	WLAN_MGMT_RESV4			0x0E	// reserved
#define 	WLAN_MGMT_RESV5			0x0F 	// reserved

// Control frame subtypes
#define 	WLAN_CTRL_RESV1			0x00	// reserved
#define 	WLAN_CTRL_RESV2			0x01	// reserved
#define 	WLAN_CTRL_RESV3			0x02	// reserved
#define 	WLAN_CTRL_RESV4			0x03	// reserved
#define 	WLAN_CTRL_RESV5			0x04	// reserved
#define 	WLAN_CTRL_RESV6			0x05	// reserved
#define 	WLAN_CTRL_RESV7			0x06	// reserved
#define 	WLAN_CTRL_RESV8			0x07	// reserved
#define 	WLAN_CTRL_RESV9			0x08	// reserved
#define 	WLAN_CTRL_RESV10		0x09	// reserved
#define 	WLAN_CTRL_PSPOLL		0x0A	// power save (PS) poll
#define 	WLAN_CTRL_RTS			0x0B	// request to send (RTS)
#define 	WLAN_CTRL_CTS			0x0C	// clear to send (CTS)
#define 	WLAN_CTRL_ACK			0x0D	// acknowledgement
#define 	WLAN_CTRL_CFEND			0x0E	// contention free end (CF) end
#define 	WLAN_CTRL_CFEND_ACK		0x0F	// CF end + CF Ack

// Data frame subtypes
#define 	WLAN_DATA_DATA			0x00	// data
#define 	WLAN_DATA_DACK			0x01	// data + cf-ack
#define 	WLAN_DATA_DPOLL			0x02	// data + cf-poll
#define 	WLAN_DATA_DACKPOLL		0x03	// data + cf-ack + cf-poll
#define 	WLAN_DATA_NULL			0x04	// null function (no data)
#define 	WLAN_DATA_ACK			0x05	// cf-ack (no data)
#define 	WLAN_DATA_POLL			0x06	// cf-poll (no data)
#define 	WLAN_DATA_ACKPOLL		0x07	// cf-ack + cf-poll (no data)
#define 	WLAN_DATA_RESV1			0x08	// reserved
#define 	WLAN_DATA_RESV2			0x09	// reserved
#define 	WLAN_DATA_RESV3			0x0A	// reserved
#define 	WLAN_DATA_RESV4			0x0B	// reserved
#define 	WLAN_DATA_RESV5			0x0C	// reserved
#define 	WLAN_DATA_RESV6			0x0D	// reserved
#define 	WLAN_DATA_RESV7			0x0E	// reserved
#define 	WLAN_DATA_RESV8			0x0F	// reserved

// Status codes
#define 	WLAN_STAT_SUCS			0x00	// success
#define 	WLAN_STAT_UNSPEC_FL		0x01	// unspecified failure
#define 	WLAN_STAT_RESV1			0x02	// reserved
#define 	WLAN_STAT_RESV2			0x03	// reserved
#define 	WLAN_STAT_RESV3			0x04	// reserved
#define 	WLAN_STAT_RESV4			0x05	// reserved
#define 	WLAN_STAT_RESV5			0x06	// reserved
#define 	WLAN_STAT_RESV6			0x07	// reserved
#define 	WLAN_STAT_RESV7			0x08	// reserved
#define 	WLAN_STAT_RESV8			0x09	// reserved
#define 	WLAN_STAT_NOSUPP		0x0A	// cannot support all req cap in cap info field
#define 	WLAN_STAT_REAS_DEN		0x0B	// reassociation denied coz cannot confirm ass exists
#define 	WLAN_STAT_AS_DEN		0x0C	// association denied due to reas out of scope of std
#define 	WLAN_STAT_AUTH_UNSUPP		0x0D	// resp station does not support spec auth algo
#define 	WLAN_STAT_AUTH_BADSEQ		0x0E	// bad auth trans sequence in auth frame
#define 	WLAN_STAT_AUTH_CHFAIL		0x0F	// auth rej becoz of challenge failure
#define 	WLAN_STAT_AUTH_TMOUT		0x10	// auth rej becoz of timeout waiting for next frame in seq
#define 	WLAN_STAT_AS_FULL		0x11	// ass rej coz AP unable to handle any more ass
#define 	WLAN_STAT_AS_BADBSS		0x12	// ass rej due to req sta not supp all BSS rates
#define 	WLAN_STAT_AS_BADSHPRE		0x13	// ass rej coz sta does not support short preamble 
#define 	WLAN_STAT_AS_BADPBCC		0x14	// ass rej coz sta does not support PBCC modulation opt
#define 	WLAN_STAT_AS_BADCHAGL		0x15	// ass rej coz req sta does not supp channel agility opt
#define 	WLAN_STAT_RESV9			0x16	// 0x16 to 0xffff are reserved !

// Information elements
#define 	WLAN_IE_SSID		0x00	// SSID	
#define 	WLAN_IE_SUPPR		0x01	// supported rates
#define 	WLAN_IE_FHSET		0x02	// FH parameter set
#define 	WLAN_IE_DSSET		0x03	// DS parameter set
#define 	WLAN_IE_CFSET		0x04	// CF parameter set
#define 	WLAN_IE_TIM		0x05	// TIM
#define 	WLAN_IE_IBSSSET		0x06	// IBSS Set
#define 	WLAN_IE_RESV1		0x07	// reserved
#define 	WLAN_IE_RESV2		0x08	// reserved
#define 	WLAN_IE_RESV3		0x09	// reserved
#define 	WLAN_IE_RESV4		0x0A	// reserved
#define 	WLAN_IE_RESV5		0x0B	// reserved
#define 	WLAN_IE_RESV6		0x0C	// reserved
#define 	WLAN_IE_RESV7		0x0D	// reserved
#define 	WLAN_IE_RESV8		0x0E	// reserved
#define 	WLAN_IE_RESV9		0x0F	// reserved
#define 	WLAN_IE_CHALTXT		0x10	// challenge text
#define 	WLAN_IE_RESV10		0x11	// 0x11 to 0x1F reserved for challenge text extension
#define 	WLAN_IE_RESV11		0x20	// 0x20 to 0xFF reserved

// Reason codes
#define 	WLAN_RSN_RESV1		0x00	// reserved
#define 	WLAN_RSN_UNSPEC		0x01	// unspecified reason
#define 	WLAN_RSN_AUTH_NOVAL	0x02	// previous authentication no longer valid
#define 	WLAN_RSN_DEAUTH		0x03	// deauthenticated becoz sending sta is leaving/left IBSS/ESS
#define 	WLAN_RSN_DISAS_INAC	0x04	// disassociated due to inactivity
#define 	WLAN_RSN_DISAS_FULL	0x05	// disassocited becoz AP is unable to handle all assoc so far
#define 	WLAN_RSN_2REC_UNAUTH	0x06	// class 2 frame received from unauth sta
#define 	WLAN_RSN_3REC_UNAS	0x07	// class 3 frame received from unass sta
#define 	WLAN_RSN_DISAS		0x08	// disassociated becoz sending sta is leaving/left BSS
#define 	WLAN_RSN_ASB4AUTH	0x09	// sta req (re)ass is not auth with responding sta
#define 	WLAN_RSN_RESV2		0x0A	// 0x0A to 0xFFFF are reserved

// 802.11 data structures

// standard 1 address frame
typedef struct _p80211_1a_t {
	__u16	fc;
	__u16	duration_id;
	__u8	mac1[WLAN_ADDR_LEN];
	__u16	seq_ctl;
	
}p80211_1a_t;

// standard 3 address frame
typedef struct _p80211_2a_t {
	__u16	fc;
	__u16	duration_id;
	__u8	mac1[WLAN_ADDR_LEN];
	__u8	mac2[WLAN_ADDR_LEN];
	__u16	seq_ctl;
	
}p80211_2a_t;

// standard 3 address frame
typedef struct _p80211_3a_t {
	__u16	fc;
	__u16	duration_id;
	__u8	mac1[WLAN_ADDR_LEN];
	__u8	mac2[WLAN_ADDR_LEN];
	__u8	mac3[WLAN_ADDR_LEN];
	__u16	seq_ctl;
	
}p80211_3a_t;

// standard 4 address frame
typedef struct _p80211_4a_t {
	__u16	fc;
	__u16	duration_id;
	__u8	mac1[WLAN_ADDR_LEN];
	__u8	mac2[WLAN_ADDR_LEN];
	__u8	mac3[WLAN_ADDR_LEN];
	__u16	seq_ctl;
	__u8	mac4[WLAN_ADDR_LEN];
	
}p80211_4a_t;

typedef union _p80211_hdr_t {
	p80211_1a_t	hdr1;
	p80211_2a_t	hdr2;
	p80211_3a_t	hdr3;
	p80211_4a_t	hdr4;	
}p80211_hdr_t;

#define GET_FVERSION(f)	((__u16)(f->hdr1.fc & 0x0003)		)
#define GET_FTYPE(f)	((__u16)(f->hdr1.fc & 0x000c) >> 2	)
#define GET_FSUBTYPE(f)	((__u16)(f->hdr1.fc & 0x00f0) >> 4	)
#define GET_FTODS(f)	((__u16)(f->hdr1.fc & 0x0100) >> 8	)
#define GET_FFROMDS(f)	((__u16)(f->hdr1.fc & 0x0200) >> 9	)
#define GET_FMFRAGS(f)	((__u16)(f->hdr1.fc & 0x0400) >> 10	)
#define GET_FRETRY(f)	((__u16)(f->hdr1.fc & 0x0800) >> 11	)
#define GET_FPMGMT(f)	((__u16)(f->hdr1.fc & 0x1000) >> 12	)
#define GET_FMDATA(f)	((__u16)(f->hdr1.fc & 0x2000) >> 13	)
#define GET_FWEP(f)	((__u16)(f->hdr1.fc & 0x4000) >> 14	)
#define GET_FORDER(f)	((__u16)(f->hdr1.fc & 0x8000) >> 15	)


typedef struct _linked_info_element {
	gen_llist ll;
	
	__u8	id;
	__u8	length;
	unsigned char *data;
}linked_info_element;

#define 	MAX_IE_DATA_LEN		256
typedef struct _info_element {
	__u8	id;
	__u8	length;
	unsigned char data[MAX_IE_DATA_LEN];
}info_element;

#define WLAN_BEACON_FIXED_PARAM_LEN 	12
#define WLAN_BEACON_TIMS_OFFSET		0
#define WLAN_BEACON_BI_OFFSET		8
#define WLAN_BEACON_CAP_OFFSET		10
#define WLAN_BEACON_CAP_WEP_MASK	0x0010

#endif /* __P80211_H_ */
